//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/BeamDistributionItem.h
//! @brief     Defines class BeamDistributionItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_ITEM_BEAMDISTRIBUTIONITEM_H
#define BORNAGAIN_GUI_MODEL_ITEM_BEAMDISTRIBUTIONITEM_H

#include "GUI/Model/Group/SelectionProperty.h"
#include "GUI/Model/Item/DistributionItems.h"
#include "Param/Distrib/ParameterDistribution.h"

//! The BeamDistributionItem handles wavelength, inclination and azimuthal parameter
//! distribution for BeamItem

class BeamDistributionItem {
public:
    virtual ~BeamDistributionItem() = default;

    virtual void serialize(Serializer& s) = 0;

    std::unique_ptr<ParameterDistribution>
    getParameterDistributionForName(ParameterDistribution::WhichParameter which) const;

    virtual double meanValue() const;
    void resetToValue(double value);

    virtual double scaleFactor() const;

    template <typename T> T* setDistributionType();
    DistributionItem* distribution() const;
    SelectionDescriptor<DistributionItem*> distributionSelection() const;
    void setDistribution(DistributionItem* d);

protected:
    virtual std::unique_ptr<IDistribution1D> createDistribution1D() const;

    SelectionProperty<DistributionItem*> m_distribution;
};

template <typename T> T* BeamDistributionItem::setDistributionType()
{
    m_distribution.set<T>();
    return dynamic_cast<T*>(m_distribution.get());
}

#endif // BORNAGAIN_GUI_MODEL_ITEM_BEAMDISTRIBUTIONITEM_H
