#include "Base/Const/Units.h"
#include "Device/Detector/IDetector2D.h"
#include "Device/Resolution/ConvolutionDetectorResolution.h"
#include "Device/Resolution/ResolutionFunction2DGaussian.h"
#include "GUI/Model/Item/InstrumentItems.h"
#include "GUI/Model/Item/RectangularDetectorItem.h"
#include "GUI/Model/Item/ResolutionFunctionItems.h"
#include "GUI/Model/Item/SphericalDetectorItem.h"
#include "GUI/Model/Types/Unit.h"
#include "Tests/GTestWrapper/google_test.h"

class TestDetectorItems : public ::testing::Test {
};

TEST_F(TestDetectorItems, resolutionFunction)
{
    InstrumentItems model;
    auto* instrument = model.addInstrument<GISASInstrumentItem>();

    DetectorItem* detectorItem = instrument->detector();

    detectorItem->setResolutionFunctionType<ResolutionFunction2DGaussianItem>();

    auto detector = detectorItem->createDetector();
    const auto* convol =
        dynamic_cast<const ConvolutionDetectorResolution*>(detector->detectorResolution());
    const auto* gaussian =
        dynamic_cast<const ResolutionFunction2DGaussian*>(convol->getResolutionFunction2D());

    EXPECT_EQ(Units::rad2deg(gaussian->sigmaX()), 0.02);
}

TEST_F(TestDetectorItems, resolutionFunctionUnit)
{
    const auto asString = [](const std::variant<QString, Unit>& unit) {
        return std::holds_alternative<QString>(unit) ? std::get<QString>(unit)
                                                     : unitAsString(std::get<Unit>(unit));
    };

    RectangularDetectorItem detector;
    EXPECT_EQ(asString(detector.resolutionFunctionSelection().currentItem()->unit()), "mm");
    detector.setResolutionFunctionType<ResolutionFunction2DGaussianItem>();
    EXPECT_EQ(asString(detector.resolutionFunctionSelection().currentItem()->unit()), "mm");
    detector.setResolutionFunctionType<ResolutionFunctionNoneItem>();
    EXPECT_EQ(asString(detector.resolutionFunctionSelection().currentItem()->unit()), "mm");

    SphericalDetectorItem sphericalDetector;
    EXPECT_EQ(asString(sphericalDetector.resolutionFunctionSelection().currentItem()->unit()),
              unitAsString(Unit::degree));
    sphericalDetector.setResolutionFunctionType<ResolutionFunction2DGaussianItem>();
    EXPECT_EQ(asString(sphericalDetector.resolutionFunctionSelection().currentItem()->unit()),
              unitAsString(Unit::degree));
    sphericalDetector.setResolutionFunctionType<ResolutionFunctionNoneItem>();
    EXPECT_EQ(asString(sphericalDetector.resolutionFunctionSelection().currentItem()->unit()),
              unitAsString(Unit::degree));
}
